@@ -1,7 +1,7 @@
 """Extensible set of components for the PsychoPy Builder view
 """
 # Part of the PsychoPy library
-# Copyright (C) 2013 Jonathan Peirce
+# Copyright (C) 2014 Jonathan Peirce
 # Distributed under the terms of the GNU General Public License (GPL).
 
 import os, glob, copy
@@ -10,7 +10,11 @@ try:
     from PIL import Image
 except ImportError:
     import Image
-from os.path import *
+from os.path import join, dirname, abspath
+
+excludeComponents = ['VisualComponent', 'BaseComponent', #these are templates, not for use
+                     'EyetrackerComponent', #this one isn't ready yet
+                     ]
 
 def pilToBitmap(pil,scaleFactor=1.0):
     image = wx.EmptyImage(pil.size[0], pil.size[1] )
@@ -20,38 +24,38 @@ def pilToBitmap(pil,scaleFactor=1.0):
     return image.ConvertToBitmap()#wx.Image and wx.Bitmap are different
 
 def getIcons(filename=None):
-        """Creates wxBitmaps ``self.icon`` and ``self.iconAdd`` based on the the image.
-        The latter has a plus sign added over the top.
+    """Creates wxBitmaps ``self.icon`` and ``self.iconAdd`` based on the the image.
+    The latter has a plus sign added over the top.
 
-        png files work best, but anything that wx.Image can import should be fine
-        """
-        icons={}
-        if filename==None:
-            filename=join(dirname(abspath(__file__)),'base.png')
-        im = Image.open(filename)
-        icons['48'] = pilToBitmap(im)
-        icons['24'] = pilToBitmap(im, scaleFactor=0.5)
-        #add the plus sign
-        add = Image.open(join(dirname(abspath(__file__)),'add.png'))
-        im.paste(add, [0,0,add.size[0], add.size[1]], mask=add)
-        #im.paste(add, [im.size[0]-add.size[0], im.size[1]-add.size[1],im.size[0], im.size[1]], mask=add)
-        icons['48add'] = pilToBitmap(im)
-        icons['24add'] = pilToBitmap(im, scaleFactor=0.5)
+    png files work best, but anything that wx.Image can import should be fine
+    """
+    icons={}
+    if filename is None:
+        filename=join(dirname(abspath(__file__)),'base.png')
+    im = Image.open(filename)
+    icons['48'] = pilToBitmap(im)
+    icons['24'] = pilToBitmap(im, scaleFactor=0.5)
+    #add the plus sign
+    add = Image.open(join(dirname(abspath(__file__)),'add.png'))
+    im.paste(add, [0,0,add.size[0], add.size[1]], mask=add)
+    #im.paste(add, [im.size[0]-add.size[0], im.size[1]-add.size[1],im.size[0], im.size[1]], mask=add)
+    icons['48add'] = pilToBitmap(im)
+    icons['24add'] = pilToBitmap(im, scaleFactor=0.5)
 
-        return icons
+    return icons
 
-def getComponents(folder=None):
+def getComponents(folder=None, fetchIcons=True):
     """Get a dictionary of available component objects for the Builder experiments.
 
-    If folder==None then the built-in components will be returned, otherwise
+    If folder is None then the built-in components will be returned, otherwise
     the components found in the folder provided will be returned.
     """
-    if folder==None:
+    if folder is None:
         folder = dirname(__file__)
     os.sys.path.append(folder)
     components={}
     #setup a default icon
-    if 'default' not in icons.keys():
+    if fetchIcons and 'default' not in icons.keys():
         icons['default']=getIcons(filename=None)
     #go through components in directory
     if os.path.isdir(folder):
@@ -65,13 +69,15 @@ def getComponents(folder=None):
                 name=None
                 #just fetch the attributes that end with 'Component', not other functions
                 if attrib.endswith('omponent') and \
-                    attrib not in ['VisualComponent', 'BaseComponent']:#must be a component
+                    attrib not in excludeComponents:#must be a component
                     name=attrib
                     components[attrib]=getattr(module, attrib)
                     #also try to get an iconfile
-                    if hasattr(module,'iconFile'):
-                        icons[name]=getIcons(module.iconFile)
-                    else:icons[name]=icons['default']
+                    if fetchIcons:
+                        if hasattr(module,'iconFile'):
+                            icons[name]=getIcons(module.iconFile)
+                        else:
+                            icons[name]=icons['default']
                     if hasattr(module, 'tooltip'):
                         tooltips[name] = module.tooltip
                     #assign the module categories to the Component
@@ -79,7 +85,7 @@ def getComponents(folder=None):
                         components[attrib].categories=['Custom']
     return components
 
-def getAllComponents(folderList=[]):
+def getAllComponents(folderList=[], fetchIcons=True):
     """Get a dictionary of all available components, from the builtins as well
     as all folders in the folderlist.
 
@@ -87,7 +93,7 @@ def getAllComponents(folderList=[]):
     """
     if type(folderList)!=list:
         raise TypeError, 'folderList should be a list, not a string'
-    components=getComponents()#get the built-ins
+    components=getComponents(fetchIcons=fetchIcons)#get the built-ins
     for folder in folderList:
         userComps=getComponents(folder)
         for thisKey in userComps.keys():
@@ -127,10 +133,12 @@ def getInitVals(params):
         elif name in ['pos', 'fieldPos']:
             inits[name].val='[0,0]'
             inits[name].valType='code'
-        elif name in ['ori','sf','size','height','letterHeight','color','phase','opacity',
-            'volume', #sounds
-            'coherence','nDots', 'fieldSize','dotSize', 'dotLife', 'dir', 'speed',#dots
-            ]:
+        elif name in ['ori','sf','size','height','letterHeight',
+                    'color','lineColor','fillColor',
+                    'phase','opacity',
+                    'volume', #sounds
+                    'coherence','nDots', 'fieldSize','dotSize', 'dotLife', 'dir', 'speed',#dots
+                    ]:
             inits[name].val="1.0"
             inits[name].valType='code'
         elif name in ['image','mask']:
@@ -149,7 +157,7 @@ def getInitVals(params):
             inits[name].val="norm"
             inits[name].valType='str'
         elif name == 'text':
-            inits[name].val="nonsense"
+            inits[name].val="default text"
             inits[name].valType='str'
         elif name == 'flip':
             inits[name].val=""
@@ -160,5 +168,6 @@ def getInitVals(params):
         else:
             print "I don't know the appropriate default value for a '%s' parameter. Please email the mailing list about this error" %name
     return inits
+
 tooltips = {}
-icons={}
+icons = {}
