@@ -1,5 +1,5 @@
 # Part of the PsychoPy library
-# Copyright (C) 2013 Jonathan Peirce
+# Copyright (C) 2014 Jonathan Peirce
 # Distributed under the terms of the GNU General Public License (GPL).
 
 import wx, copy
@@ -7,19 +7,53 @@ from os import path
 from psychopy.app.builder.experiment import Param
 from psychopy.constants import *
 
+# only use _localized values for label values, nothing functional
+_localized = {'name': _translate('Name'),  # fieldName: display label
+              'startType': _translate('start type'), 'stopType': _translate('stop type'),
+              'startVal': _translate('Start'), 'stopVal': _translate('Stop'),
+              'startEstim': _translate('Expected start (s)'), 'durationEstim': _translate('Expected duration (s)')
+             }
+
 class BaseComponent(object):
     """A template for components, defining the methods to be overridden"""
     #override the categories property below
     categories = ['Custom']#an attribute of the class, determines the section in the components panel
-    def __init__(self, exp, parentName, name=''):
+    def __init__(self, exp, parentName, name='',
+                startType='time (s)', startVal='',
+                stopType='duration (s)', stopVal='',
+                startEstim='', durationEstim=''):
         self.type='Base'
         self.exp=exp#so we can access the experiment if necess
         self.parentName=parentName#to access the routine too if needed
+
         self.params={}
         self.params['name']=Param(name, valType='code',
-            hint="Name of this component",
-            label="Name")
-        self.order=['name']#make name come first (others don't matter)
+            hint=_translate("Name of this component (alpha-numeric or _, no spaces)"),
+            label=_localized['name'])
+        self.params['startType']=Param(startType, valType='str',
+            allowedVals=['time (s)', 'frame N', 'condition'],
+            hint=_translate("How do you want to define your start point?"),
+            label=_localized['startType'])
+        self.params['stopType']=Param(stopType, valType='str',
+            allowedVals=['duration (s)', 'duration (frames)', 'time (s)', 'frame N', 'condition'],
+            hint=_translate("How do you want to define your end point?"),
+            label=_localized['stopType'])
+        self.params['startVal']=Param(startVal, valType='code', allowedTypes=[],
+            hint=_translate("When does the component start?"),
+            label=_localized['startVal'])
+        self.params['stopVal']=Param(stopVal, valType='code', allowedTypes=[],
+            updates='constant', allowedUpdates=[],
+            hint=_translate("When does the component end? (blank is endless)"),
+            label=_localized['stopVal'])
+        self.params['startEstim']=Param(startEstim, valType='code', allowedTypes=[],
+            hint=_translate("(Optional) expected start (s), purely for representing in the timeline"),
+            label=_localized['startEstim'])
+        self.params['durationEstim']=Param(durationEstim, valType='code', allowedTypes=[],
+            hint=_translate("(Optional) expected duration (s), purely for representing in the timeline"),
+            label=_localized['durationEstim'])
+
+        self.order=['name']  # name first, then timing, then others
+
     def writeStartCode(self,buff):
         """Write any code that a component needs that should only ever be done at
         start of an experiment (done once only)
@@ -79,43 +113,73 @@ class BaseComponent(object):
         """Test whether we need to stop
         """
         if self.params['stopType'].val=='time (s)':
-            buff.writeIndented("elif %(name)s.status == STARTED and t >= %(stopVal)s:\n" %(self.params))
+            buff.writeIndented("if %(name)s.status == STARTED and t >= (%(stopVal)s-win.monitorFramePeriod*0.75): #most of one frame period left\n" %(self.params))
         #duration in time (s)
         elif self.params['stopType'].val=='duration (s)' and self.params['startType'].val=='time (s)':
-            buff.writeIndented("elif %(name)s.status == STARTED and t >= (%(startVal)s + %(stopVal)s):\n" %(self.params))
+            buff.writeIndented("if %(name)s.status == STARTED and t >= (%(startVal)s + (%(stopVal)s-win.monitorFramePeriod*0.75)): #most of one frame period left\n" %(self.params))
         elif self.params['stopType'].val=='duration (s)':#start at frame and end with duratio (need to use approximate)
-            buff.writeIndented("elif %(name)s.status == STARTED and t >= (%(name)s.tStart + %(stopVal)s):\n" %(self.params))
+            buff.writeIndented("if %(name)s.status == STARTED and t >= (%(name)s.tStart + %(stopVal)s):\n" %(self.params))
         #duration in frames
         elif self.params['stopType'].val=='duration (frames)':
-            buff.writeIndented("elif %(name)s.status == STARTED and frameN >= (%(name)s.frameNStart + %(stopVal)s):\n" %(self.params))
+            buff.writeIndented("if %(name)s.status == STARTED and frameN >= (%(name)s.frameNStart + %(stopVal)s):\n" %(self.params))
         #stop frame number
         elif self.params['stopType'].val=='frame N':
-            buff.writeIndented("elif %(name)s.status == STARTED and frameN >= %(stopVal)s:\n" %(self.params))
+            buff.writeIndented("if %(name)s.status == STARTED and frameN >= %(stopVal)s:\n" %(self.params))
         #end according to a condition
         elif self.params['stopType'].val=='condition':
-            buff.writeIndented("elif %(name)s.status == STARTED and (%(stopVal)s):\n" %(self.params))
+            buff.writeIndented("if %(name)s.status == STARTED and (%(stopVal)s):\n" %(self.params))
         else:
             raise "Didn't write any stop line for startType=%(startType)s, stopType=%(stopType)s" %(self.params)
         buff.setIndentLevel(+1,relative=True)
 
-    def writeParamUpdates(self, buff, updateType):
+    def writeParamUpdates(self, buff, updateType, paramNames=None):
         """write updates to the buffer for each parameter that needs it
         updateType can be 'experiment', 'routine' or 'frame'
         """
-        #add this once all stimulus setXXX() methods have an autoLog argument
-#        if updateType=='frame':
-#            logStr = ", autoLog=False"
-#            logComment="#updating too often to be worth logging"
-#        else:
-#            logStr = logComment=""
-
-        for thisParamName in self.params.keys():
+        if paramNames is None:
+            paramNames = self.params.keys()
+        for thisParamName in paramNames:
+            if thisParamName=='advancedParams':
+                continue # advancedParams is not really a parameter itself
             thisParam=self.params[thisParamName]
             if thisParam.updates==updateType:
-                if thisParamName=='color':
-                    paramCaps=self.params['colorSpace'].upper() #setRGB, not setColor
-                else:paramCaps=thisParamName.capitalize()
-                buff.writeIndented("%s.set%s(%s)\n" %(self.params['name'],paramCaps, thisParam) )
+                self.writeParamUpdate(buff, self.params['name'],
+                    thisParamName, thisParam, thisParam.updates)
+    def writeParamUpdate(self, buff, compName, paramName, val, updateType, params=None):
+        """Writes an update string for a single parameter.
+        This should not need overriding for different components - try to keep
+        constant
+        """
+        if params is None:
+            params=self.params
+        #first work out the name for the set____() function call
+        if paramName=='advancedParams':
+            return # advancedParams is not really a parameter itself
+        elif paramName=='letterHeight':
+            paramCaps='Height' #setHeight for TextStim
+        elif paramName=='image' and self.getType()=='PatchComponent':
+            paramCaps='Tex' #setTex for PatchStim
+        elif paramName=='sf':
+            paramCaps='SF' #setSF, not SetSf
+        elif paramName=='coherence':
+            paramCaps='FieldCoherence'
+        elif paramName=='fieldPos':
+            paramCaps='FieldPos'
+        else:
+            paramCaps = paramName[0].capitalize() + paramName[1:]
+        #then write the line
+        if updateType=='set every frame':
+            loggingStr = ', log=False'
+        else:
+            loggingStr = ''
+        #write the line
+        if paramName=='color':
+            buff.writeIndented("%s.setColor(%s, colorSpace=%s" \
+                %(compName, params['color'], params['colorSpace']))
+            buff.write("%s)\n" %(loggingStr))
+        else:
+            buff.writeIndented("%s.set%s(%s%s)\n" %(compName, paramCaps, val, loggingStr))
+
     def checkNeedToUpdate(self, updateType):
         """Determine whether this component has any parameters set to repeat at this level
 
@@ -141,7 +205,7 @@ class BaseComponent(object):
         nonSlipSafe indicates that the component's duration is a known fixed
         value and can be used in non-slip global clock timing (e.g for fMRI)
         """
-        if not self.params.has_key('startType'):
+        if not 'startType' in self.params:
             return None, None, True#this component does not have any start/stop
         startType=self.params['startType'].val
         stopType=self.params['stopType'].val
@@ -179,6 +243,47 @@ class BaseComponent(object):
     def getShortType(self):
         return self.getType().replace('Component','')
 
+class UnknownComponent(BaseComponent):
+    """This is used by Builder to represent a component that was not known by the
+    current installed version of PsychoPy (most likely from the future). We want
+    this to be loaded, represented and saved but not used in any script-outputs.
+    It should have nothing but a name - other params will be added by the loader
+    """
+    def __init__(self, exp, parentName, name=''):
+        self.type='Unknown'
+        self.exp=exp#so we can access the experiment if necess
+        self.parentName=parentName#to access the routine too if needed
+        self.params={}
+        self.params['name']=Param(name, valType='code',
+            hint=_translate("Name of this component (alpha-numeric or _, no spaces)"),
+            label=_localized['name'])
+        self.order=['name']  # name first, then timing, then others
+    #make sure nothing gets written into experiment for an unknown object class!
+    def writeRoutineStartCode(self,buff):
+        pass
+    def writeStartCode(self,buff):
+        pass
+    def writeInitCode(self,buff):
+        pass
+    def writeFrameCode(self,buff):
+        pass
+    def writeRoutineStartCode(self,buff):
+        pass
+    def writeRoutineEndCode(self,buff):
+        pass
+    def writeExperimentEndCode(self,buff):
+        pass
+    def writeTimeTestCode(self,buff):
+        pass
+    def writeStartTestCode(self,buff):
+        pass
+    def writeStopTestCode(self,buff):
+        pass
+    def writeParamUpdates(self, buff, updateType, paramNames=None):
+        pass
+    def writeParamUpdate(self, buff, compName, paramName, val, updateType, params=None):
+        pass
+
 def canBeNumeric(inStr):
     """Determines whether the input can be converted to a float
     (using a try: float(instr))
@@ -187,4 +292,4 @@ def canBeNumeric(inStr):
         float(inStr)
         return True
     except:
-        return False
\ No newline at end of file
+        return False
