@@ -1,9 +1,9 @@
-#!/usr/bin/env python
+#!/usr/bin/env python2
 from psychopy.hardware import cedrus
 from psychopy import core
 import sys
 
-rb730 = cedrus.RB730(7,'RB730',baudrate=115200)
+rb730 = cedrus.RB730(7,baudrate=115200)
 #get RB info
 print rb730.getInfo()
 print 'roundTrip:', rb730.measureRoundTrip()#this is the time taken to send a signal to the unit and back via USB
@@ -21,4 +21,4 @@ while notAbort:
         print evt.key, evt.rt, evt.direction
         if evt.key==1:notAbort=False
 print 'done'
-print 'baseTime:',rb730.getBaseTimer()
\ No newline at end of file
+print 'baseTime:',rb730.getBaseTimer()
