@@ -1,16 +1,29 @@
 """To build simple dialogues etc. (requires wxPython)
 """
 # Part of the PsychoPy library
-# Copyright (C) 2013 Jonathan Peirce
+# Copyright (C) 2014 Jonathan Peirce
 # Distributed under the terms of the GNU General Public License (GPL).
+
 from psychopy import logging
-#from wxPython import wx
 import wx
 import numpy
 import string, os
+from psychopy.app import localization
 
 OK = wx.ID_OK
 
+def ensureWxApp():
+    # make sure there's a wxApp prior to showing a gui, e.g., for expInfo dialog
+    try:
+        wx.Dialog(None, -1)  # not shown; FileDialog gives same exception
+        return True
+    except wx._core.PyNoAppError:
+        if wx.version() < '2.9':
+            return wx.PySimpleApp()
+        else:
+            return wx.App(False)
+
+
 class Dlg(wx.Dialog):
     """A simple dialogue box. You can add text or input boxes
     (sequentially) and then retrieve the values.
@@ -28,22 +41,22 @@ class Dlg(wx.Dialog):
         myDlg.addText('Experiment Info')
         myDlg.addField('Grating Ori:',45)
         myDlg.addField('Group:', choices=["Test", "Control"])
-        myDlg.show()#show dialog and wait for OK or Cancel
-        if gui.OK:#then the user pressed OK
+        myDlg.show()  # show dialog and wait for OK or Cancel
+        if myDlg.OK:  # then the user pressed OK
             thisInfo = myDlg.data
             print thisInfo
-        else: print 'user cancelled'
+        else:
+            print 'user cancelled'
     """
-    def __init__(self,title='PsychoPy dialogue',
+    def __init__(self,title=_translate('PsychoPy dialogue'),
             pos=None, size=wx.DefaultSize,
-            style=wx.DEFAULT_DIALOG_STYLE|wx.DIALOG_NO_PARENT):
+            style=wx.DEFAULT_DIALOG_STYLE|wx.DIALOG_NO_PARENT,
+            labelButtonOK = _translate(" OK "),
+            labelButtonCancel = _translate(" Cancel ")):
         style=style|wx.RESIZE_BORDER
-        try:
-            wx.Dialog.__init__(self, None,-1,title,pos,size,style)
-        except:
-            global app
-            app = wx.PySimpleApp()
-            wx.Dialog.__init__(self, None,-1,title,pos,size,style)
+        global app  # avoid recreating for every gui
+        app = ensureWxApp()
+        wx.Dialog.__init__(self, None,-1,title,pos,size,style)
         self.inputFields = []
         self.inputFieldTypes= []
         self.inputFieldNames= []
@@ -52,8 +65,16 @@ class Dlg(wx.Dialog):
         self.sizer = wx.BoxSizer(wx.VERTICAL)
         #self.addText('')#insert some space at top of dialogue
         self.pos = pos
+        self.labelButtonOK = labelButtonOK
+        self.labelButtonCancel = labelButtonCancel
     def addText(self, text, color=''):
-        textLength = wx.Size(8*len(text)+16, 25)
+        # the horizontal extent can depend on the locale and font in use:
+        font = self.GetFont()
+        dc = wx.WindowDC(self)
+        dc.SetFont(font)
+        textWidth, textHeight = dc.GetTextExtent(text)
+        textLength = wx.Size(textWidth + 50, textHeight)
+
         myTxt = wx.StaticText(self,-1,
                                 label=text,
                                 style=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_CENTER_HORIZONTAL,
@@ -79,7 +100,7 @@ class Dlg(wx.Dialog):
         labelLength = wx.Size(9*len(label)+16,25)#was 8*until v0.91.4
         inputLabel = wx.StaticText(self,-1,label,
                                         size=labelLength,
-                                        style=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
+                                        style=wx.ALIGN_RIGHT)
         if len(color): inputLabel.SetForegroundColour(color)
         container.Add(inputLabel, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
         #create input control
@@ -90,7 +111,7 @@ class Dlg(wx.Dialog):
             inputLength = wx.Size(max(50, 5*len(unicode(initial))+16), 25)
             inputBox = wx.TextCtrl(self,-1,unicode(initial),size=inputLength)
         else:
-            inputBox = wx.Choice(self, -1, choices=[str(option) for option in list(choices)])
+            inputBox = wx.Choice(self, -1, choices=[unicode(option) for option in list(choices)])
             # Somewhat dirty hack that allows us to treat the choice just like
             # an input box when retrieving the data
             inputBox.GetValue = inputBox.GetStringSelection
@@ -124,16 +145,16 @@ class Dlg(wx.Dialog):
         """
         #add buttons for OK and Cancel
         buttons = wx.BoxSizer(wx.HORIZONTAL)
-        OK = wx.Button(self, wx.ID_OK, " OK ")
+        OK = wx.Button(self, wx.ID_OK, self.labelButtonOK)
         OK.SetDefault()
 
         buttons.Add(OK)
-        CANCEL = wx.Button(self, wx.ID_CANCEL, " Cancel ")
+        CANCEL = wx.Button(self, wx.ID_CANCEL, self.labelButtonCancel)
         buttons.Add(CANCEL)
         self.sizer.Add(buttons,1,flag=wx.ALIGN_RIGHT|wx.ALIGN_BOTTOM,border=5)
 
         self.SetSizerAndFit(self.sizer)
-        if self.pos == None:
+        if self.pos is None:
             self.Center()
         if self.ShowModal() == wx.ID_OK:
             self.data=[]
@@ -162,7 +183,6 @@ class Dlg(wx.Dialog):
         #self.myApp.Exit()
 
 
-
 class DlgFromDict(Dlg):
     """Creates a dialogue box that represents a dictionary of values.
     Any values changed by the user are change (in-place) by this
@@ -189,6 +209,7 @@ class DlgFromDict(Dlg):
     """
     def __init__(self, dictionary, title='',fixed=[], order=[], tip={}):
         Dlg.__init__(self, title)
+        #app = ensureWxApp() done by Dlg
         self.dictionary=dictionary
         keys = self.dictionary.keys()
         keys.sort()
@@ -207,23 +228,22 @@ class DlgFromDict(Dlg):
             else:
                 self.addField(field,self.dictionary[field], tip=tooltip)
         #show it and collect data
-        #tmp= wx.PySimpleApp()#this should have been done by Dlg ?
         self.show()
         if self.OK:
             for n,thisKey in enumerate(keys):
                 self.dictionary[thisKey]=self.data[n]
 
 def fileSaveDlg(initFilePath="", initFileName="",
-                prompt="Select file to save",
+                prompt=_translate("Select file to save"),
                 allowed=None):
-    """A simple dialogue allowing access to the file system.
+    """A simple dialogue allowing write access to the file system.
     (Useful in case you collect an hour of data and then try to
     save to a non-existent directory!!)
 
     :parameters:
         initFilePath: string
             default file path on which to open the dialog
-        initFilePath: string
+        initFileName: string
             default file name, as suggested file
         prompt: string (default "Select file to open")
             can be set to custom prompts
@@ -237,18 +257,14 @@ def fileSaveDlg(initFilePath="", initFileName="",
 
     If user cancels the None is returned.
     """
-    if allowed==None:
+    if allowed is None:
         allowed = "All files (*.*)|*.*"  #\
             #"txt (*.txt)|*.txt" \
             #"pickled files (*.pickle, *.pkl)|*.pickle" \
             #"shelved files (*.shelf)|*.shelf"
-    try:
-        dlg = wx.FileDialog(None,prompt, 
-                          initFilePath, initFileName, allowed, wx.SAVE)
-    except:
-        tmpApp = wx.PySimpleApp()
-        dlg = wx.FileDialog(None,prompt, 
-                          initFilePath, initFileName, allowed, wx.SAVE)
+    global app  # avoid recreating for every gui
+    app = ensureWxApp()
+    dlg = wx.FileDialog(None,prompt, initFilePath, initFileName, allowed, wx.SAVE)
     if dlg.ShowModal() == OK:
         #get names of images and their directory
         outName = dlg.GetFilename()
@@ -261,16 +277,14 @@ def fileSaveDlg(initFilePath="", initFileName="",
 
 def fileOpenDlg(tryFilePath="",
                 tryFileName="",
-                prompt="Select file to open",
+                prompt=_translate("Select file to open"),
                 allowed=None):
-    """A simple dialogue allowing access to the file system.
-    (Useful in case you collect an hour of data and then try to
-    save to a non-existent directory!!)
+    """A simple dialogue allowing read access to the file system.
 
     :parameters:
         tryFilePath: string
             default file path on which to open the dialog
-        tryFilePath: string
+        tryFileName: string
             default file name, as suggested file
         prompt: string (default "Select file to open")
             can be set to custom prompts
@@ -284,19 +298,16 @@ def fileOpenDlg(tryFilePath="",
 
     If user cancels, then None is returned.
     """
-    if allowed==None:
+    if allowed is None:
         allowed = "PsychoPy Data (*.psydat)|*.psydat|"\
             "txt (*.txt,*.dlm,*.csv)|*.txt;*.dlm;*.csv|" \
             "pickled files (*.pickle, *.pkl)|*.pickle|" \
             "shelved files (*.shelf)|*.shelf|" \
             "All files (*.*)|*.*"
-    try:
-        dlg = wx.FileDialog(None, prompt,
-                          tryFilePath, tryFileName, allowed, wx.OPEN|wx.FILE_MUST_EXIST|wx.MULTIPLE)
-    except:
-        tmpApp = wx.PySimpleApp()
-        dlg = wx.FileDialog(None, prompt,
-                          tryFilePath, tryFileName, allowed, wx.OPEN|wx.FILE_MUST_EXIST|wx.MULTIPLE)
+    global app  # avoid recreating for every gui
+    app = ensureWxApp()
+    dlg = wx.FileDialog(None, prompt, tryFilePath, tryFileName, allowed,
+                        wx.OPEN|wx.FILE_MUST_EXIST|wx.MULTIPLE)
     if dlg.ShowModal() == OK:
         #get names of images and their directory
         fullPaths = dlg.GetPaths()
