@@ -1,10 +1,7 @@
 import sys, glob, collections
 from itertools import imap,chain
-
 from psychopy import logging
-__all__=['forp','cedrus','minolta','pr', 'crs', 'ioLabs']
-
-
+__all__=['forp','cedrus','minolta','pr', 'crs', 'iolab']
 
 
 def getSerialPorts():
@@ -20,6 +17,7 @@ def getSerialPorts():
             '/dev/tty.Key*', #some are Keyspan.1 or Keyserial.1
             '/dev/tty.modem*',
             '/dev/cu.usbmodem*', #for PR650
+            '/dev/tty.usbserial*', #for the 'Plugable' converter, according to Tushar Chauhan
         ]
     elif sys.platform.startswith("linux"):
         ports = [
@@ -40,7 +38,7 @@ def getSerialPorts():
         logging.error("We don't support serial ports on {0} yet!"
                       .format(sys.platform))
         return []
-    
+
     # This creates an iterator for each glob expression. The glob
     # expressions are then chained together. This is more efficient
     # because it means we don't perform the lookups before we actually
@@ -48,7 +46,7 @@ def getSerialPorts():
     return chain.from_iterable(imap(glob.iglob,ports))
 
 def getAllPhotometers():
-    """Gets all available photometers. 
+    """Gets all available photometers.
     The returned photometers may vary depending on which drivers are installed.
     Standalone PsychoPy ships with libraries for all supported photometers.
 
@@ -63,14 +61,14 @@ def getAllPhotometers():
     return photometers
 
 def getPhotometerByName(name):
-    """Gets a Photometer class by name. 
+    """Gets a Photometer class by name.
     You can use either short names like pr650 or a long name like
     CRS ColorCAL.
-    
+
     :parameters:
         name : The name of the device
-    
-    :returns: 
+
+    :returns:
     Returns the photometer matching the passed in device
     name or none if we were unable to find it.
 
@@ -80,43 +78,38 @@ def getPhotometerByName(name):
         if name.lower() in photom.driverFor or name == photom.longName:
             return photom
 
-
-        
-
-    
-
 def findPhotometer(ports=None, device=None):
-    """Try to find a connected photometer/photospectrometer! 
-    PsychoPy will sweep a series of serial ports trying to open them. If a port 
-    successfully opens then it will try to issue a command to the device. If it 
-    responds with one of the expected values then it is assumed to be the 
-    appropriate device. 
-    
+    """Try to find a connected photometer/photospectrometer!
+    PsychoPy will sweep a series of serial ports trying to open them. If a port
+    successfully opens then it will try to issue a command to the device. If it
+    responds with one of the expected values then it is assumed to be the
+    appropriate device.
+
     :parameters:
-        
+
         ports : a list of ports to search
-            Each port can be a string (e.g. 'COM1', ''/dev/tty.Keyspan1.1') or a 
-            number (for win32 comports only). If none are provided then PsychoPy 
+            Each port can be a string (e.g. 'COM1', ''/dev/tty.Keyspan1.1') or a
+            number (for win32 comports only). If none are provided then PsychoPy
             will sweep COM0-10 on win32 and search known likely port names on OS X
             and linux.
-            
+
         device : string giving expected device (e.g. 'PR650', 'PR655', 'LS110').
-            If this is not given then an attempt will be made to find a device of 
+            If this is not given then an attempt will be made to find a device of
             any type, but this often fails
-            
+
     :returns:
-    
+
         * An object representing the first photometer found
         * None if the ports didn't yield a valid response
         * None if there were not even any valid ports (suggesting a driver not being installed)
-        
+
     e.g.::
-    
+
         photom = findPhotometer(device='PR655') #sweeps ports 0 to 10 searching for a PR655
         print photom.getLum()
         if hasattr(photom, 'getSpectrum'):#can retrieve spectrum (e.g. a PR650)
             print photom.getSpectrum()
-        
+
     """
     if isinstance(device,basestring):
         photometers = [getPhotometerByName(device)]
@@ -126,13 +119,13 @@ def findPhotometer(ports=None, device=None):
     else:
         photometers = getAllPhotometers()
 
-    
+
     #determine candidate ports
-    if ports == None: 
+    if ports is None:
         ports = getSerialPorts()
     elif type(ports) in [int,float] or isinstance(ports,basestring):
         ports=[ports] #so that we can iterate
-        
+
     #go through each port in turn
     photom=None
     logging.info('scanning serial ports...')
@@ -148,18 +141,18 @@ def findPhotometer(ports=None, device=None):
             except Exception as ex:
                 logging.error("Couldn't initialize photometer {0}: {1}".format(Photometer.__name__,ex))
                 continue # We threw an exception so we should just skip ahead
-            if photom.OK: 
+            if photom.OK:
                 logging.info(' ...found a %s\n' %(photom.type)); logging.flush()
                 #we're now sure that this is the correct device and that it's configured
                 #now increase the number of attempts made to communicate for temperamental devices!
                 if hasattr(photom,'setMaxAttempts'):photom.setMaxAttempts(10)
                 return photom#we found one so stop looking
             else:
-                if photom.com and photom.com.isOpen: 
+                if photom.com and photom.com.isOpen:
                     logging.info('closing port')
                     photom.com.close()
 
         #If we got here we didn't find one
         logging.info('...nope!\n\t'); logging.flush()
-            
+
     return None
