@@ -38,29 +38,29 @@ cores in your machine or, when your machine has many of them (e.g. > 4),
 perhaps one less than this.  < S. Simpson Note: These are 'not' GIL bound
 threads and therefore actually improve performance > """
 
-DATA_FILE_TITLE="ioHub DataStore - Experiment Data File."
-FILE_VERSION = '0.7.0'
-SCHEMA_AUTHORS='Sol Simpson'
-SCHEMA_MODIFIED_DATE='May 6th, 2013'
+DATA_FILE_TITLE = "ioHub DataStore - Experiment Data File."
+FILE_VERSION = '0.8.0.2'
+SCHEMA_AUTHORS = 'Sol Simpson'
+SCHEMA_MODIFIED_DATE = 'May 4th, 2014'
 
         
 class ioHubpyTablesFile():
     
-    def __init__(self,fileName,folderPath,fmode='a',ioHubsettings=None):
-        self.fileName=fileName
-        self.folderPath=folderPath
-        self.filePath=os.path.join(folderPath,fileName)
+    def __init__(self, fileName, folderPath, fmode='a', ioHubsettings=None):
+        self.fileName = fileName
+        self.folderPath = folderPath
+        self.filePath = os.path.join(folderPath, fileName)
 
-        self.settings=ioHubsettings
+        self.settings = ioHubsettings
 
-        self.active_experiment_id=None
-        self.active_session_id=None
+        self.active_experiment_id = None
+        self.active_session_id = None
         
-        self.flushCounter=self.settings.get('flush_interval',32)
-        self._eventCounter=0
+        self.flushCounter = self.settings.get('flush_interval', 32)
+        self._eventCounter = 0
         
-        self.TABLES=dict()
-        self._eventGroupMappings=dict()
+        self.TABLES = dict()
+        self._eventGroupMappings = dict()
         self.emrtFile = openFile(self.filePath, mode = fmode)
                
         atexit.register(close_open_data_files, False)
@@ -99,12 +99,13 @@ class ioHubpyTablesFile():
         
         # create tables dict of hdf5 path mappings
 
+
         try:
             self.TABLES['KEYBOARD_KEY']=self.emrtFile.root.data_collection.events.keyboard.KeyboardKeyEvent
         except:
             # Just means the table for this event type has not been created as the event type is not being recorded
             pass
-        
+
         try:
             self.TABLES['KEYBOARD_CHAR']=self.emrtFile.root.data_collection.events.keyboard.KeyboardCharEvent
         except:
@@ -117,6 +118,12 @@ class ioHubpyTablesFile():
             # Just means the table for this event type has not been created as the event type is not being recorded
             pass
 
+        try:
+            self.TABLES['TOUCH']=self.emrtFile.root.data_collection.events.touch.TouchEvent
+        except:
+            # Just means the table for this event type has not been created as the event type is not being recorded
+            pass
+
         try:
             self.TABLES['GAMEPAD_STATE_CHANGE']=self.emrtFile.root.data_collection.events.gamepad.GamepadStateChangeEvent
         except:
@@ -141,6 +148,36 @@ class ioHubpyTablesFile():
             # Just means the table for this event type has not been created as the event type is not being recorded
             pass
 
+        try:
+            self.TABLES['ANALOG_INPUT']=self.emrtFile.root.data_collection.events.mcu.AnalogInputEvent
+        except:
+            # Just means the table for this event type has not been created as the event type is not being recorded
+            pass
+
+        try:
+            self.TABLES['THRESHOLD']=self.emrtFile.root.data_collection.events.mcu.ThresholdEvent
+        except:
+            # Just means the table for this event type has not been created as the event type is not being recorded
+            pass
+
+        try:
+            self.TABLES['DIGITAL_INPUT']=self.emrtFile.root.data_collection.events.mcu.DigitalInputEvent
+        except:
+            # Just means the table for this event type has not been created as the event type is not being recorded
+            pass
+
+        try:
+            self.TABLES['SERIAL_INPUT'] = self.emrtFile.root.data_collection.events.serial.SerialInputEvent
+        except:
+            # Just means the table for this event type has not been created as the event type is not being recorded
+            pass
+
+        try:
+            self.TABLES['SERIAL_BYTE_CHANGE'] = self.emrtFile.root.data_collection.events.serial.SerialByteChangeEvent
+        except:
+            # Just means the table for this event type has not been created as the event type is not being recorded
+            pass
+
         try:
             self.TABLES['MONOCULAR_EYE_SAMPLE']=self.emrtFile.root.data_collection.events.eyetracker.MonocularEyeSampleEvent
         except:
@@ -218,9 +255,12 @@ class ioHubpyTablesFile():
         self.emrtFile.createGroup(self.emrtFile.root.data_collection.events, 'experiment', title='Experiment Device Events.')
         self.emrtFile.createGroup(self.emrtFile.root.data_collection.events, 'keyboard', title='Keyboard Device Events.')
         self.emrtFile.createGroup(self.emrtFile.root.data_collection.events, 'mouse', title='Mouse Device Events.')
+        self.emrtFile.createGroup(self.emrtFile.root.data_collection.events, 'touch', title='Touch Device Events.')
         self.emrtFile.createGroup(self.emrtFile.root.data_collection.events, 'gamepad', title='GamePad Device Events.')
         self.emrtFile.createGroup(self.emrtFile.root.data_collection.events, 'analog_input', title='AnalogInput Device Events.')
         self.emrtFile.createGroup(self.emrtFile.root.data_collection.events, 'eyetracker', title='EyeTracker Device Events.')
+        self.emrtFile.createGroup(self.emrtFile.root.data_collection.events, 'mcu', title='MCU Device Events.')
+        self.emrtFile.createGroup(self.emrtFile.root.data_collection.events, 'serial', title='Serial Interface Events.')
         self.flush()
 
         self._buildEventGroupMappingDict()
@@ -230,8 +270,14 @@ class ioHubpyTablesFile():
         self._eventGroupMappings['KEYBOARD_KEY']=self.emrtFile.root.data_collection.events.keyboard
         self._eventGroupMappings['KEYBOARD_CHAR']=self.emrtFile.root.data_collection.events.keyboard
         self._eventGroupMappings['MOUSE_INPUT']=self.emrtFile.root.data_collection.events.mouse
+        self._eventGroupMappings['TOUCH']=self.emrtFile.root.data_collection.events.touch
         self._eventGroupMappings['GAMEPAD_STATE_CHANGE']=self.emrtFile.root.data_collection.events.gamepad
         self._eventGroupMappings['MULTI_CHANNEL_ANALOG_INPUT']=self.emrtFile.root.data_collection.events.analog_input
+        self._eventGroupMappings['ANALOG_INPUT']=self.emrtFile.root.data_collection.events.mcu
+        self._eventGroupMappings['THRESHOLD']=self.emrtFile.root.data_collection.events.mcu
+        self._eventGroupMappings['DIGITAL_INPUT']=self.emrtFile.root.data_collection.events.mcu
+        self._eventGroupMappings['SERIAL_INPUT']=self.emrtFile.root.data_collection.events.serial
+        self._eventGroupMappings['SERIAL_BYTE_CHANGE']=self.emrtFile.root.data_collection.events.serial
         self._eventGroupMappings['MESSAGE']=self.emrtFile.root.data_collection.events.experiment
         self._eventGroupMappings['LOG']=self.emrtFile.root.data_collection.events.experiment
         self._eventGroupMappings['MONOCULAR_EYE_SAMPLE']=self.emrtFile.root.data_collection.events.eyetracker
@@ -296,8 +342,12 @@ class ioHubpyTablesFile():
         #ioHub.print2err("Session ID set to: ",self.active_session_id)
         return self.active_session_id
 
-    def _initializeConditionVariableTable(self,experiment_id,np_dtype):
+    def _initializeConditionVariableTable(self,experiment_id,session_id,np_dtype):
         experimentConditionVariableTable=None
+        exp_session=[('EXPERIMENT_ID','i4'),('SESSION_ID','i4')]
+        exp_session.extend(np_dtype)
+        np_dtype=exp_session
+        #print2err('np_dtype: ',np_dtype,' ',type(np_dtype))
         self._EXP_COND_DTYPE=N.dtype(np_dtype)
         try:
             expCondTableName="EXP_CV_%d"%(experiment_id)
@@ -318,10 +368,14 @@ class ioHubpyTablesFile():
         self._activeRunTimeConditionVariableTable=experimentConditionVariableTable
         return True
 
-    def _addRowToConditionVariableTable(self,session_id,data):
+    def _addRowToConditionVariableTable(self,experiment_id,session_id,data):
         if self.emrtFile and 'EXP_CV' in self.TABLES and self._EXP_COND_DTYPE is not None:
+            temp=[experiment_id,session_id]
+            temp.extend(data)
+            data=temp            
             try:
                 etable=self.TABLES['EXP_CV']
+                #print2err('data: ',data,' ',type(data))
 
                 for i,d in enumerate(data):
                     if isinstance(d,(list,tuple)):
@@ -460,15 +514,19 @@ class ioHubpyTablesFile():
 
 def close_open_data_files(verbose):
     open_files = tables.file._open_files
-    are_open_files = len(open_files) > 0
-    if verbose and are_open_files:
-        print "Closing remaining open data files:"
-    for fileh in open_files.keys():
-        if verbose:
-            print "%s..." % (open_files[fileh].filename,)
-        open_files[fileh].close()
-        if verbose:
-            print "done"
+    clall = hasattr(open_files,'close_all')
+    if clall:
+        open_files.close_all()
+    else:
+        are_open_files = len(open_files) > 0
+        if verbose and are_open_files:
+            print2err("Closing remaining open data files:")
+        for fileh in open_files.keys():
+            if verbose:
+                print2err( "%s..." % (open_files[fileh].filename,))
+            open_files[fileh].close()
+            if verbose:
+                print2err("done")
 
 try:
     global registered_close_open_data_files
