@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+from __future__ import division
 """
 ioHub
 .. file: ioHub/server.py
@@ -16,14 +18,25 @@ import os,sys
 from operator import itemgetter
 from collections import deque
 import psychopy.iohub
-from psychopy.iohub import OrderedDict,print2err, printExceptionDetailsToStdErr, ioHubError, createErrorResult,convertCamelToSnake, DeviceConstants,EventConstants,Computer, DeviceEvent, import_device, IO_HUB_DIRECTORY, load, dump, Loader, Dumper
+from psychopy.iohub import OrderedDict, convertCamelToSnake, IO_HUB_DIRECTORY
+from psychopy.iohub import load, dump, Loader, Dumper
+from psychopy.iohub import print2err, printExceptionDetailsToStdErr, ioHubError
+from psychopy.iohub import DeviceConstants, EventConstants
+from psychopy.iohub import Computer, DeviceEvent, import_device
 from psychopy.iohub.devices.deviceConfigValidation import validateDeviceConfiguration
 from psychopy.iohub.net import MAX_PACKET_SIZE
 currentSec= Computer.currentSec
 
 import json
 import msgpack
-    
+try:
+    import msgpack_numpy as m
+    m.patch()
+except:
+    pass
+
+import psutil
+
 class udpServer(DatagramServer):
     def __init__(self,ioHubServer,address,coder='msgpack'):
         self.iohub=ioHubServer
@@ -43,11 +56,13 @@ class udpServer(DatagramServer):
         if self._running is False:
             return False
         
-        self.feed(request[:-2])
+        self.feed(request)
         request = self.unpack()   
         request_type= request.pop(0)
-        
-        if request_type == 'PING':
+        if request_type == 'SYNC_REQ':
+            self.sendResponse(['SYNC_REPLY',currentSec()],replyTo)  
+            return True        
+        elif request_type == 'PING':
                 clienttime=request.pop(0)
                 msg_id=request.pop(0)
                 payload=request.pop(0)
@@ -71,10 +86,9 @@ class udpServer(DatagramServer):
             try:
                 result=getattr(self,callable_name)
             except:
-                self.sendResponse(createErrorResult('RPC_ATTRIBUTE_ERROR',
-                                        msg="The method name referenced could not be found by the RPC server.",
-                                        method_name=callable_name),
-                                    replyTo)
+                print2err("RPC_ATTRIBUTE_ERROR")
+                printExceptionDetailsToStdErr()
+                self.sendResponse('RPC_ATTRIBUTE_ERROR',replyTo)
                 return False
                 
             if result and callable(result):
@@ -92,20 +106,14 @@ class udpServer(DatagramServer):
                     self.sendResponse(edata,replyTo)
                     return True
                 except Exception,e:
-                    self.sendResponse(createErrorResult('RPC_RUNTIME_ERROR',
-                                      msg="An error occurred on the ioHub Server while evaulating an RPC request",
-                                      method_name=callable_name,
-                                      args=args,
-                                      kwargs=kwargs,
-                                      exception=str(e))
-                                ,replyTo)
+                    print2err("RPC_RUNTIME_ERROR")
+                    printExceptionDetailsToStdErr()
+                    self.sendResponse('RPC_RUNTIME_ERROR', replyTo)
                     return False
             else:
-                self.sendResponse(createErrorResult('RPC_NOT_CALLABLE_ERROR',
-                                    msg="The method name give is not callable (it is not a method).",
-                                    method_name=callable_name,
-                                    resolved_result=str(result))
-                                ,replyTo)
+                print2err("RPC_NOT_CALLABLE_ERROR")
+                printExceptionDetailsToStdErr()
+                self.sendResponse('RPC_NOT_CALLABLE_ERROR', replyTo)
                 return False
         elif request_type == 'STOP_IOHUB_SERVER':
             try:
@@ -113,10 +121,9 @@ class udpServer(DatagramServer):
             except:
                 printExceptionDetailsToStdErr
         else:
-            self.sendResponse(createErrorResult('RPC_TYPE_NOT_SUPPORTED_ERROR',
-                                    msg="The request type provided is not recognized by the ioHub Server.",
-                                    request_type=request_type),
-                                replyTo)
+            print2err("RPC_NOT_CALLABLE_ERROR")
+            printExceptionDetailsToStdErr()
+            self.sendResponse('RPC_NOT_CALLABLE_ERROR', replyTo)
             return False
             
     def handleGetEvents(self,replyTo):
@@ -125,16 +132,15 @@ class udpServer(DatagramServer):
             self.iohub.eventBuffer.clear()
 
             if len(currentEvents)>0:
-                sorted(currentEvents, key=itemgetter(DeviceEvent.EVENT_HUB_TIME_INDEX))
+                currentEvents=sorted(currentEvents, key=itemgetter(DeviceEvent.EVENT_HUB_TIME_INDEX))
                 self.sendResponse(('GET_EVENTS_RESULT',currentEvents),replyTo)
             else:
                 self.sendResponse(('GET_EVENTS_RESULT', None),replyTo)
             return True
-        except Exception,e:
-            self.sendResponse(createErrorResult('IOHUB_GET_EVENTS_ERROR',
-                                    msg="An error occurred while events were being retrived from the ioHub Server",
-                                    exception=str(e)),
-                                replyTo)
+        except Exception, e:
+            print2err("IOHUB_GET_EVENTS_ERROR")
+            printExceptionDetailsToStdErr()
+            self.sendResponse('IOHUB_GET_EVENTS_ERROR', replyTo)
             return False
 
     def handleExperimentDeviceRequest(self,request,replyTo):
@@ -168,18 +174,18 @@ class udpServer(DatagramServer):
                 dev=ioServer.deviceDict.get(dclass,None)
             
             if dev is None:
-                self.sendResponse(createErrorResult('IOHUB_DEVICE_ERROR',
-                                        msg="An instance of the ioHub Device class provided is not enabled on the ioHub Server",
-                                        device_class=dclass),
-                                    replyTo)                
+                print2err("IOHUB_DEVICE_ERROR")
+                printExceptionDetailsToStdErr()
+                self.sendResponse('IOHUB_DEVICE_ERROR', replyTo)
                 return False
+
             
             try:
                 method=getattr(dev,dmethod)
             except:
-                self.sendResponse(createErrorResult('IOHUB_DEVICE_METHOD_ERROR',
-                                        msg="Device class {0} does not have a method called {1}".format(dclass,dmethod)),
-                                    replyTo)                
+                print2err("IOHUB_DEVICE_METHOD_ERROR")
+                printExceptionDetailsToStdErr()
+                self.sendResponse('IOHUB_DEVICE_METHOD_ERROR', replyTo)
                 return False
                 
             result=[]
@@ -195,15 +201,11 @@ class udpServer(DatagramServer):
                 self.sendResponse(('DEV_RPC_RESULT',result),replyTo)
                 return True
             except Exception, e:
-                self.sendResponse(createErrorResult('RPC_DEVICE_RUNTIME_ERROR',
-                                      msg="An error occurred on the ioHub Server while evaulating an Device RPC request",
-                                      device=dclass,
-                                      dmethod=dmethod,
-                                      args=args,
-                                      kwargs=kwargs,
-                                      exception=str(e))
-                                ,replyTo)
+                print2err("RPC_DEVICE_RUNTIME_ERROR")
+                printExceptionDetailsToStdErr()
+                self.sendResponse('RPC_DEVICE_RUNTIME_ERROR', replyTo)
                 return False
+
         elif request_type == 'GET_DEVICE_LIST':
             try:            
                 dev_list=[]
@@ -212,14 +214,10 @@ class udpServer(DatagramServer):
                 self.sendResponse(('GET_DEV_LIST_RESULT',len(dev_list),dev_list),replyTo)
                 return True
             except Exception, e:
+                print2err("RPC_DEVICE_RUNTIME_ERROR")
                 printExceptionDetailsToStdErr()
-                self.sendResponse(createErrorResult('RPC_DEVICE_RUNTIME_ERROR',
-                                      msg="An error occurred on the ioHub Server while getting the Device list for the Experiment Process",
-                                      devices=str(self.iohub.devices),
-                                      dev_list=str(dev_list),
-                                      exception=str(e))
-                                ,replyTo)
-                return False                
+                self.sendResponse('RPC_DEVICE_RUNTIME_ERROR', replyTo)
+                return False
 
         elif request_type == 'GET_DEV_INTERFACE':
             dclass=request.pop(0)
@@ -238,11 +236,11 @@ class udpServer(DatagramServer):
                 self.sendResponse(('GET_DEV_INTERFACE',data),replyTo)
                 return True
             else:
-                self.sendResponse(createErrorResult('GET_DEV_INTERFACE_ERROR',
-                                        msg="An error occurred on the ioHub Server while retrieving device interface information.",
-                                        device=dclass),
-                                  replyTo)
+                print2err("GET_DEV_INTERFACE_ERROR")
+                printExceptionDetailsToStdErr()
+                self.sendResponse('GET_DEV_INTERFACE_ERROR', replyTo)
                 return False
+
         elif request_type == 'ADD_DEVICE':
             dclass_name=request.pop(0)
             dconfig_dict=request.pop(1)
@@ -255,27 +253,28 @@ class udpServer(DatagramServer):
                 self.sendResponse(('ADD_DEVICE',data),replyTo)
                 return True
             else:
-                self.sendResponse(createErrorResult('ADD_DEVICE_ERROR',
-                                        msg="An error occurred on the ioHub Server while adding a device to be monitored.",
-                                        device=dclass_name,
-                                        config=dconfig_dict),
-                                  replyTo)
-            return False
+
+                print2err("ADD_DEVICE_ERROR")
+                printExceptionDetailsToStdErr()
+                self.sendResponse('ADD_DEVICE_ERROR', replyTo)
+                return False
         else:
-            self.sendResponse(createErrorResult('DEVICE_RPC_TYPE_NOT_SUPPORTED_ERROR',
-                                    msg="The device RPC request type provided is not recognized by the ioHub Server.",
-                                    request_type=request_type),
-                              replyTo)
+            print2err("DEVICE_RPC_TYPE_NOT_SUPPORTED_ERROR")
+            printExceptionDetailsToStdErr()
+            self.sendResponse('DEVICE_RPC_TYPE_NOT_SUPPORTED_ERROR', replyTo)
             return False
             
     def sendResponse(self,data,address):
         packet_data=None
         try:
-            max_size=MAX_PACKET_SIZE/2-20
-            packet_data=self.pack(data)+'\r\n'
-            packet_data_length=len(packet_data)
-            if packet_data_length>= max_size:
-                num_packets=len(packet_data)/max_size+1
+            num_packets = -1
+            packet_data_length = -1
+            # TODO: Max packet size on OS X seems to be 8192 !!
+            max_size = int(MAX_PACKET_SIZE/2-20)
+            packet_data = self.pack(data)
+            packet_data_length = len(packet_data)
+            if packet_data_length >= max_size:
+                num_packets = int(packet_data_length//max_size)+1
                 self.sendResponse(('IOHUB_MULTIPACKET_RESPONSE',num_packets),address)
                 for p in xrange(num_packets-1):
                     self.socket.sendto(packet_data[p*max_size:(p+1)*max_size],address)
@@ -284,8 +283,11 @@ class udpServer(DatagramServer):
                 self.socket.sendto(packet_data,address)
         except:
             print2err('Error trying to send data to experiment process:')
-            print2err('data length:',len(data))
-            print2err("=============================")            
+            print2err('max_size: ',max_size)
+            print2err('data length: ',packet_data_length)
+            print2err('num_packets: ',num_packets)
+
+            print2err("=============================")
             printExceptionDetailsToStdErr()
             print2err("=============================")            
 
@@ -302,13 +304,9 @@ class udpServer(DatagramServer):
                 packet_data_length=len(packet_data)
                 print2err('packet Data length: ',len(packet_data))
 
-            data=createErrorResult('IOHUB_SERVER_RESPONSE_ERROR',       
-                                   msg="The ioHub Server Failed to send the intended response.",
-                                   first_data_element=str(first_data_element),
-                                   packet_data_length=packet_data_length,
-                                   max_packet_size=max_size)
-            packet_data=self.pack(data)+'\r\n'
-            packet_data_length=len(packet_data)            
+            print2err("IOHUB_SERVER_RESPONSE_ERROR")
+            printExceptionDetailsToStdErr()
+            packet_data=self.pack('IOHUB_SERVER_RESPONSE_ERROR')
             self.socket.sendto(packet_data,address)
             
     def setExperimentInfo(self,experimentInfoList):
@@ -327,6 +325,20 @@ class udpServer(DatagramServer):
             return False
         except:
             printExceptionDetailsToStdErr()
+
+    def registerPygletWindowHandles(self,*win_hwhds):
+        if self.iohub:
+            for wh in win_hwhds:
+                if wh not in self.iohub._pyglet_window_hnds:
+                    self.iohub._pyglet_window_hnds.append(wh)            
+            #print2err(">>IOHUB.registerPygletWindowHandles:",win_hwhds)
+        
+    def unregisterPygletWindowHandles(self,*win_hwhds):
+        if self.iohub:
+            for wh in win_hwhds:
+                if wh in self.iohub._pyglet_window_hnds:
+                    self.iohub._pyglet_window_hnds.remove(wh)
+            #print2err("<<IOHUB.unregisterPygletWindowHandles:",win_hwhds)
             
     def createExperimentSessionEntry(self,sessionInfoDict):
         self.iohub.sessionInfoDict=sessionInfoDict
@@ -339,7 +351,7 @@ class udpServer(DatagramServer):
 
     def initializeConditionVariableTable(self, experiment_id, session_id, numpy_dtype):
         if self.iohub.emrt_file:
-            output=[('session_id','u4'),('index_id','u4')]
+            output=[]
             for a in numpy_dtype:
                 if isinstance(a[1],(str,unicode)):
                     output.append(tuple(a))
@@ -349,22 +361,22 @@ class udpServer(DatagramServer):
                         temp[1].append(tuple(i))
                     output.append(tuple(temp))
 
-            return self.iohub.emrt_file._initializeConditionVariableTable(experiment_id,output)
+            return self.iohub.emrt_file._initializeConditionVariableTable(experiment_id,session_id,output)
         return False
 
-    def addRowToConditionVariableTable(self,session_id,data):
+    def addRowToConditionVariableTable(self,experiment_id,session_id,data):
         if self.iohub.emrt_file:
-            return self.iohub.emrt_file._addRowToConditionVariableTable(session_id,data)
+            return self.iohub.emrt_file._addRowToConditionVariableTable(experiment_id,session_id,data)
         return False
 
     def clearEventBuffer(self):
         return self.iohub.clearEventBuffer()
 
     def enableHighPriority(self,disable_gc=True):
-        Computer.enableHighPriority(disable_gc)
+        return Computer.enableHighPriority(disable_gc)
 
     def disableHighPriority(self):
-        Computer.disableHighPriority()
+        return Computer.disableHighPriority()
 
     def getProcessAffinity(self):
         return Computer.getCurrentProcessAffinity()
@@ -404,18 +416,23 @@ class DeviceMonitor(Greenlet):
             self.device._poll()
             i=self.sleep_interval-(ctime()-stime)
             if i > 0.0:
-                gevent.sleep(i)    
+                gevent.sleep(i)
+            else:
+                gevent.sleep(0.0)
+                
+    def __del__(self):
+        self.device = None
 
-        
 class ioServer(object):
     eventBuffer=None
     deviceDict={}
     _logMessageBuffer=deque(maxlen=128)
+    _pyglet_window_hnds=[]
     def __init__(self, rootScriptPathDir, config=None):
         self._session_id=None
         self._experiment_id=None
 
-        self.log("Server Time Offset: {0}".format(Computer.globalClock.getLastResetTime()))
+        self.log("Server Time Offset: {0}".format(Computer.global_clock.getLastResetTime()))
 
         self._hookManager=None
         self.emrt_file=None
@@ -472,6 +489,23 @@ class ioServer(object):
             raise ioHubError("Error during device creation ....")
 
 
+        # Add PubSub device listeners to other event types
+        try:
+            for d in self.devices:
+                if d.__class__.__name__ == "EventPublisher":
+                    monitored_event_ids=d._event_listeners.keys()
+                    for eid in monitored_event_ids:
+                        event_device_class=EventConstants.getClass(eid).PARENT_DEVICE
+                        for ed in self.devices:
+                            if ed.__class__ == event_device_class:
+                                ed._addEventListener(d,[eid,])
+                                break
+                            
+        except Exception, e:
+            print2err("Error PubSub Device listener association ....")
+            printExceptionDetailsToStdErr()
+            raise e
+
         # initial time offset
         #print2err("-- ioServer Init Complete -- ")
         
@@ -587,9 +621,15 @@ class ioServer(object):
                             if pythoncom.PumpWaitingMessages() == 1:
                                 raise KeyboardInterrupt()               
         
+                        def __del__(self):
+                            #if self._mouseHooked and self._hookManager:
+                            #    self._hookManager.UnhookMouse()
+                            #elif self._keyboardHooked and self._hookManager:
+                            #    self._hookManager.UnhookKeyboard()
+                            self._hookManager=None
                     #print2err("Creating pyHook Monitor......")
                     self._hookDevice=pyHookDevice()
-                    hookMonitor=DeviceMonitor(self._hookDevice,0.00375)
+                    hookMonitor=DeviceMonitor(self._hookDevice, self.config.get('windows_msgpump_interval', 0.00375))
                     self.deviceMonitors.append(hookMonitor)
                 
                     #print2err("Created pyHook Monitor.")
@@ -614,7 +654,8 @@ class ioServer(object):
                 from .devices import pyXHook
                 if self._hookManager is None:
                     #iohub.log("Creating pyXHook Monitors....")
-                    self._hookManager = pyXHook.HookManager()
+                    log_events = self.config.get('log_raw_kb_mouse_events',False)
+                    self._hookManager = pyXHook.HookManager(log_events)
                     self._hookManager._mouseHooked=False
                     self._hookManager._keyboardHooked=False
 
@@ -747,28 +788,28 @@ class ioServer(object):
                 dPoller=DeviceMonitor(deviceInstance,interval)
                 self.deviceMonitors.append(dPoller)
 
-            eventIDs=[]
+            monitoringEventIDs=[]
             monitor_events_list=device_config.get('monitor_event_types',[])
             if isinstance(monitor_events_list,(list,tuple)):
                 for event_class_name in monitor_events_list:
-                    eventIDs.append(getattr(EventConstants,convertCamelToSnake(event_class_name[:-5],False)))
-            
-            self.log("{0} Instance Event IDs To Monitor: {1}".format(device_class_name,eventIDs))
+                    event_id = getattr(EventConstants,convertCamelToSnake(event_class_name[:-5],False))
+                    monitoringEventIDs.append(event_id)
+            self.log("{0} Instance Event IDs To Monitor: {1}".format(device_class_name,monitoringEventIDs))
             #ioHub.print2err("{0} Instance Event IDs To Monitor: {1}".format(device_class_name,eventIDs))
 
             # add event listeners for streaming events
             if device_config.get('stream_events') is True:
                 self.log("Online event access is being enabled for: %s"%device_class_name)
                 # add listener for global event queue
-                deviceInstance._addEventListener(self,eventIDs)
+                deviceInstance._addEventListener(self,monitoringEventIDs)
                 #ioHub.print2err("ioServer event stream listener added: device=%s eventIDs=%s"%(device_class_name,eventIDs))
-                self.log("Standard event stream listener added for ioServer for event ids %s"%(str(eventIDs),))
+                self.log("Standard event stream listener added for ioServer for event ids %s"%(str(monitoringEventIDs),))
                 # add listener for device event queue
-                deviceInstance._addEventListener(deviceInstance,eventIDs)
+                deviceInstance._addEventListener(deviceInstance,monitoringEventIDs)
                 #  ioHub.print2err("%s event stream listener added: eventIDs=%s"%(device_class_name,eventIDs))
-                self.log("Standard event stream listener added for class %s for event ids %s"%(device_class_name,str(eventIDs)))
+                self.log("Standard event stream listener added for class %s for event ids %s"%(device_class_name,str(monitoringEventIDs)))
 
-            return deviceInstance,device_config,eventIDs,event_classes
+            return deviceInstance,device_config,monitoringEventIDs,event_classes
 
 
     def log(self,text,level=None):
@@ -778,12 +819,9 @@ class ioServer(object):
             if exp and self._session_id and self._experiment_id:
                 while len(self._logMessageBuffer):
                     lm=self._logMessageBuffer.popleft()
-                    #print2err('>>>!!! Logging BACKLOGGED LogEvent: ',lm,", ",(exp, self._session_id, self._experiment_id))
                     exp.log(*lm)
-                #print2err('>>>!!! Logging LogEvent: ',(text,level,log_time),", ",(exp, self._session_id, self._experiment_id))
                 exp.log(text,level,log_time)
             else:
-                #print2err('>>>!!! Adding LogEvent to _logMessageBuffer: ',(text,level,log_time),", ",(exp, self._session_id, self._experiment_id))
                 self._logMessageBuffer.append((text,level,log_time))
         except:
             printExceptionDetailsToStdErr()
@@ -809,15 +847,26 @@ class ioServer(object):
     def _processDeviceEventIteration(self):
         for device in self.devices:
             try:
-                events=device._getNativeEventBuffer()
-                #if events and len(events)>0:
-                #    ioHub.print2err("_processDeviceEventIteration.....", device._event_listeners)
-                while len(events)>0:
-                    evt=events.popleft()
-                    e=device._getIOHubEventObject(evt)
+                events = device._getNativeEventBuffer()
+
+                while len(events) > 0:
+                    evt = events.popleft()
+                    e = device._getIOHubEventObject(evt)
                     if e is not None:
                         for l in device._getEventListeners(e[DeviceEvent.EVENT_TYPE_ID_INDEX]):
                             l._handleEvent(e)
+
+
+                filtered_events = []
+                for filter in device._filters.values():
+                    filtered_events.extend(filter._removeOutputEvents())
+
+                for i in range(len(filtered_events)):
+                    e = filtered_events[i]
+                    for l in device._getEventListeners(e[DeviceEvent.EVENT_TYPE_ID_INDEX]):
+                        l._handleEvent(e)
+
+
             except:
                 printExceptionDetailsToStdErr()
                 print2err("Error in processDeviceEvents: ", device, " : ", len(events), " : ", e)
@@ -825,7 +874,6 @@ class ioServer(object):
                 print2err("--------------------------------------")
 
     def _handleEvent(self,event):
-        #ioHub.print2err("ioServer Handle event: ",event)
         self.eventBuffer.append(event)
 
     def clearEventBuffer(self):
@@ -833,23 +881,45 @@ class ioServer(object):
         self.eventBuffer.clear()
         return l
 
+    def checkForPsychopyProcess(self, sleep_interval):
+        while self._running:
+            if Computer.psychopy_process:
+                try:
+                    if Computer.psychopy_process.is_running() is False:
+                        Computer.psychopy_process = None
+                        psychopy.iohub.MessageDialog("PsychoPy Process dead. Should shut down.")
+                        self.shutdown()
+                        sys.exit(1)
+                except:
+                        sys.exit(2)
+            gevent.sleep(sleep_interval)
+
     def shutdown(self):
         try:
             self._running=False
-    
+
             if Computer.system=='linux2':
                 if self._hookManager:
                     self._hookManager.cancel()
-    
+            elif Computer.system=='win32':
+                if self._hookDevice and self._hookDevice._hookManager:
+                    #self._hookDevice._hookManager = None #.UnhookMouse()
+                    #self._hookDevice._hookManager.UnhookKeyboard()
+                    self._hookDevice._hookManager = None
+                    self._hookDevice = None
+                    
             while len(self.deviceMonitors) > 0:
                 m=self.deviceMonitors.pop(0)
                 m.running=False
+
             if self.eventBuffer:
                 self.clearEventBuffer()
+
             try:
                 self.closeDataStoreFile()
             except:
                 pass
+
             while len(self.devices) > 0:
                 d=self.devices.pop(0)
                 try:
@@ -857,7 +927,6 @@ class ioServer(object):
                         d._close()
                 except:
                         pass
-            gevent.sleep()
         except:
             print2err("Error in ioSever.shutdown():")
             printExceptionDetailsToStdErr()
