@@ -1,5 +1,5 @@
 # Part of the PsychoPy library
-# Copyright (C) 2012 Jonathan Peirce
+# Copyright (C) 2014 Jonathan Peirce
 # Distributed under the terms of the GNU General Public License (GPL).
 
 import os, sys
@@ -12,7 +12,8 @@ if 'testMonitor' not in calibTools.getAllMonitors():
         width=30,
         distance=57,
         gamma=1.0,
-        notes='default (not very useful) monitor')
+        # can't always localize the notes easily; need app created first to import localization and use _translate( ) => issues
+        notes='default (not very useful) monitor'
+        )
     defMon.setSizePix([1024,768])
     defMon.saveMon()
-    
\ No newline at end of file
