@@ -1,7 +1,7 @@
 """Tools to help with calibrations
 """
 # Part of the PsychoPy library
-# Copyright (C) 2013 Jonathan Peirce
+# Copyright (C) 2014 Jonathan Peirce
 # Distributed under the terms of the GNU General Public License (GPL).
 
 from calibData import *
@@ -47,18 +47,6 @@ if isinstance(monitorFolder, str):
 if not os.path.isdir(monitorFolder):
     os.makedirs(monitorFolder)
 
-    #try to import monitors from old location (PsychoPy <0.93 used site-packages/monitors instead)
-    #this only gets done if there was no existing .psychopy folder (and so no calib files)
-    import glob, shutil #these are just to copy old calib files across
-    try:
-        calibFiles = glob.glob('C:\Python24\Lib\site-packages\monitors\*.calib')
-        for thisFile in calibFiles:
-            thisPath, fileName = os.path.split(thisFile)
-            shutil.copyfile(thisFile, join(monitorFolder,fileName))
-    except:
-        pass #never mind - the user will have to do it!
-
-
 pr650code={'OK':'000\r\n',#this is returned after measure
     '18':'Light Low',#these is returned at beginning of data
     '10':'Light Low',
@@ -72,7 +60,7 @@ def findPR650(ports=None):
     logging.error("DEPRECATED (as of v.1.60.01). Use psychopy.hardware.findPhotometer() instead, which "\
     +"finds a wider range of devices")
 
-    if ports==None:
+    if ports is None:
         if sys.platform=='darwin':
             ports=[]
             #try some known entries in /dev/tty. used by keyspan
@@ -82,7 +70,7 @@ def findPR650(ports=None):
             if len(ports)==0: logging.error("couldn't find likely serial port in /dev/tty.* Check for \
                 serial port name manually, check drivers installed etc...")
         elif sys.platform=='win32':
-            ports = range(11)
+            ports = range(20)
     elif type(ports) in [int,float]:
         ports=[ports] #so that we can iterate
     pr650=None
@@ -158,13 +146,14 @@ class Monitor:
         useBits=None,
         verbose=True,
         currentCalib={},
-         ):
+        autoLog=True ):
         """
         """
 
         #make sure that all necessary settings have some value
         self.__type__ = 'psychoMonitor'
         self.name = name
+        self.autoLog = autoLog
         self.currentCalib = currentCalib
         self.currentCalibName = strFromDate(time.localtime())
         self.calibs = {}
@@ -174,10 +163,11 @@ class Monitor:
         self._loadAll()
         if len(self.calibNames)>0:
             self.setCurrent(-1) #will fetch previous vals if monitor exists
+            if self.autoLog:
+                logging.info('Loaded monitor calibration from %s' %self.calibNames)
         else:
             self.newCalib()
-
-        logging.info(self.calibNames)
+            logging.warning("Monitor specification not found. Creating a temporary one...")
 
         #overide current monitor settings with the vals given
         if width: self.setWidth(width)
@@ -189,12 +179,10 @@ class Monitor:
     def gammaIsDefault(self):
         """Determine whether we're using the default gamma values
         """
-        thisGrid = self.getGammaGrid()
-        #the old default (for simple gamma eq)
-        oldGrid = numpy.array([[0,1,1],[0,1,1],[0,1,1],[0,1,1]])
+        thisGamma = self.getGamma()
         #run the test just on this
-        if thisGrid==None \
-            or numpy.alltrue(thisGrid[:4,:3]==oldGrid):
+        if thisGamma is None \
+            or numpy.alltrue(numpy.array(thisGamma)==numpy.array([1,1,1])):
             return True
         else:
             return False
@@ -211,7 +199,7 @@ class Monitor:
     def setCalibDate(self, date=None):
         """Sets the calibration to a given date/time or to the current
         date/time if none given. (Also returns the date as set)"""
-        if date==None:
+        if date is None:
             date=time.localtime()
         self.currentCalib['calibDate'] = date
         return date
@@ -229,7 +217,7 @@ class Monitor:
         0 uses y=a+(bx)^gamma
         1 uses y=(a+bx)^gamma
         2 uses linear interpolation over the curve"""
-        self.currentCalib['lineariseMethod']=method
+        self.currentCalib['linearizeMethod']=method
     def setMeanLum(self, meanLum):
         """Records the mean luminance (for reference only)"""
         self.currentCalib['meanLum']=meanLum
@@ -269,7 +257,7 @@ class Monitor:
     #equivalent get functions
     def getSizePix(self):
         """Returns the size of the current calibration in pixels, or None if not defined"""
-        if self.currentCalib.has_key('sizePix'):
+        if 'sizePix' in self.currentCalib:
             return self.currentCalib['sizePix']
         else:
             return None
@@ -284,13 +272,15 @@ class Monitor:
         calibTools.strFromDate"""
         return self.currentCalib['calibDate']
     def getGamma(self):
-        if self.currentCalib.has_key('gamma'):
+        if 'gammaGrid' in self.currentCalib and not numpy.alltrue(self.getGammaGrid()[1:, 2]==1):
+            return self.getGammaGrid()[1:, 2]
+        elif 'gamma' in self.currentCalib:
             return self.currentCalib['gamma']
         else:
             return None
     def getGammaGrid(self):
         """Gets the min,max,gamma values for the each gun"""
-        if self.currentCalib.has_key('gammaGrid'):
+        if 'gammaGrid' in self.currentCalib:
             # Make sure it's an array, so you can look at the shape
             grid = numpy.asarray(self.currentCalib['gammaGrid'])
             if grid.shape!=[4,6]:
@@ -300,35 +290,37 @@ class Monitor:
             return grid
         else:
             return None
-    def getLineariseMethod(self):
+    def getLinearizeMethod(self):
         """Gets the min,max,gamma values for the each gun"""
-        if self.currentCalib.has_key('lineariseMethod'):
+        if 'linearizeMethod' in self.currentCalib:
+            return self.currentCalib['linearizeMethod']
+        elif 'lineariseMethod' in self.currentCalib:
             return self.currentCalib['lineariseMethod']
         else:
             return None
     def getMeanLum(self):
-        if self.currentCalib.has_key('meanLum'):
+        if 'meanLum' in self.currentCalib:
             return self.currentCalib['meanLum']
         else:
             return None
     def getLumsPre(self):
         """Gets the measured luminance values from last calibration"""
-        if self.currentCalib.has_key('lumsPre'):
+        if 'lumsPre' in self.currentCalib:
             return self.currentCalib['lumsPre']
         else: return None
     def getLumsPost(self):
         """Gets the measured luminance values from last calibration TEST"""
-        if self.currentCalib.has_key('lumsPost'):
+        if 'lumsPost' in self.currentCalib:
             return self.currentCalib['lumsPost']
         else: return None
     def getLevelsPre(self):
         """Gets the measured luminance values from last calibration"""
-        if self.currentCalib.has_key('levelsPre'):
+        if 'levelsPre' in self.currentCalib:
             return self.currentCalib['levelsPre']
         else: return None
     def getLevelsPost(self):
         """Gets the measured luminance values from last calibration TEST"""
-        if self.currentCalib.has_key('levelsPost'):
+        if 'levelsPost' in self.currentCalib:
             return self.currentCalib['levelsPost']
         else: return None
     def getSpectra(self):
@@ -339,7 +331,7 @@ class Monitor:
             - nm, power = monitor.getSpectra()
 
             """
-        if self.currentCalib.has_key('spectraNM'):
+        if 'spectraNM' in self.currentCalib:
             return self.currentCalib['spectraNM'], self.currentCalib['spectraRGB']
         else:
             return None, None
@@ -349,10 +341,10 @@ class Monitor:
         Otherwise, if power spectra are available for the
         monitor a matrix will be calculated.
         """
-        if not self.currentCalib.has_key('dkl_rgb'): RECOMPUTE=True
+        if not 'dkl_rgb' in self.currentCalib: RECOMPUTE=True
         if RECOMPUTE:
             nm, power = self.getSpectra()
-            if nm==None:
+            if nm is None:
                 return None
             else:
                 return makeDKL2RGB(nm, power)
@@ -365,10 +357,10 @@ class Monitor:
         Otherwise (if power spectra are available for the
         monitor) a matrix will be calculated.
         """
-        if not self.currentCalib.has_key('lms_rgb'): RECOMPUTE=True
+        if not 'lms_rgb' in self.currentCalib: RECOMPUTE=True
         if RECOMPUTE:
             nm, power = self.getSpectra()
-            if nm==None:
+            if nm is None:
                 return None
             else:
                 return makeLMS2RGB(nm, power)
@@ -394,7 +386,6 @@ class Monitor:
             self.name+".calib")     #the name of the actual file
 
         if not os.path.exists(thisFileName):
-            logging.warning("Creating new monitor...")
             self.calibNames = []
         else:
             thisFile = open(thisFileName,'r')
@@ -412,7 +403,7 @@ class Monitor:
         verbose=True):
         """create a new (empty) calibration for this monitor and
         makes this the current calibration"""
-        if calibName==None:
+        if calibName is None:
             calibName= strFromDate(time.localtime())
         #add to the list of calibrations
         self.calibNames.append(calibName)
@@ -461,8 +452,6 @@ class Monitor:
             return False
 
         self.currentCalib = self.calibs[self.currentCalibName]      #do the import
-        logging.info("Loaded calibration from:%s" %self.currentCalibName)
-
         return self.currentCalibName
 
     def delCalib(self,calibName):
@@ -486,7 +475,7 @@ class Monitor:
         """
         Stores the settings for the current calibration settings as new monitor.
         """
-        if calibName==None:
+        if calibName is None:
             calibName= strFromDate(time.localtime())
         #add to the list of calibrations
         self.calibNames.append(calibName)
@@ -496,7 +485,7 @@ class Monitor:
     def lineariseLums(self, desiredLums, newInterpolators=False, overrideGamma=None):
         """lums should be uncalibrated luminance values (e.g. a linear ramp)
         ranging 0:1"""
-        linMethod = self.getLineariseMethod()
+        linMethod = self.getLinearizeMethod()
         desiredLums = numpy.asarray(desiredLums)
         output = desiredLums*0.0 #needs same size as input
 
@@ -506,7 +495,8 @@ class Monitor:
             if self._gammaInterpolator!=None and not newInterpolators:
                 pass #we already have an interpolator
             elif lumsPre != None:
-                logging.info('Creating linear interpolation for gamma')
+                if self.autoLog:
+                    logging.info('Creating linear interpolation for gamma')
                 #we can make an interpolator
                 self._gammaInterpolator, self._gammaInterpolator2 =[],[]
                 #each of these interpolators is a function!
@@ -544,7 +534,8 @@ class Monitor:
                 else: gamma = gammaGrid[1:4,2]
                 maxLumWhite = gammaGrid[0,1]
                 gammaWhite = gammaGrid[0,2]
-                logging.debug('using gamma grid'+str(gammaGrid))
+                if self.autoLog:
+                    logging.debug('using gamma grid'+str(gammaGrid))
             else:
                 #just do the calculation using gamma
                 minLum=0
@@ -726,7 +717,8 @@ def getLumSeries(lumLevels=8,
     useBits=False,
     autoMode='auto',
     stimSize = 0.3,
-    photometer=None):
+    photometer=None,
+    screen = 0):
     """
     Automatically measures a series of gun values and measures
     the luminance with a photometer.
@@ -757,7 +749,7 @@ def getLumSeries(lumLevels=8,
     """
     import psychopy.event, psychopy.visual
     from psychopy import core
-    if photometer==None:
+    if photometer is None:
         havePhotom = False
     elif not hasattr(photometer, 'getLum'):
         logging.error("photometer argument to monitors.getLumSeries should be a type of photometer "+\
@@ -777,7 +769,7 @@ def getLumSeries(lumLevels=8,
     #setup screen and "stimuli"
     myWin = psychopy.visual.Window(fullscr = 0, size=winSize,
         gamma=gamma,units='norm',monitor=monitor,allowGUI=True,winType='pyglet',
-        bitsMode=bitsMode)
+        bitsMode=bitsMode, screen=screen)
     instructions="Point the photometer at the central bar. Hit a key when ready (or wait 30s)"
     message = psychopy.visual.TextStim(myWin, text = instructions,height=0.1,
         pos=(0,-0.85), rgb=[1,-1,-1])
@@ -798,16 +790,18 @@ def getLumSeries(lumLevels=8,
         instructions="Point the photometer at the central white bar. Hit a key when ready (or wait %iss)" %tRemain
         backPatch.draw()
         testPatch.draw()
-        message.setText(instructions)
+        message.setText(instructions, log=False)
         message.draw()
         myWin.flip()
         if len(psychopy.event.getKeys()):
             break#we got a keypress so move on
 
+    if autoMode!='semi':
+        message.setText('Q to quit at any time')
+    else:
+        message.setText('Spacebar for next patch')
 
-    message.setText('Q to quit at any time')
-    #
-    if photometer.type=='LS100':#LS100 likes to take at least one bright measurement
+    if havePhotom and photometer.type=='LS100': #LS100 likes to take at least one bright measurement
         junk=photometer.getLum()
 
     #what are the test values of luminance
@@ -817,7 +811,7 @@ def getLumSeries(lumLevels=8,
 
     if allGuns: guns=[0,1,2,3]#gun=0 is the white luminance measure
     else: allGuns=[0]
-    lumsList = numpy.zeros((len(guns),len(toTest)), 'd') #this will hoold the measured luminance values
+    lumsList = numpy.zeros((len(guns),len(toTest)), 'd') #this will hold the measured luminance values
     #for each gun, for each value run test
     for gun in guns:
         for valN, DACval in enumerate(toTest):
@@ -838,11 +832,7 @@ def getLumSeries(lumLevels=8,
             myWin.flip()
 
             time.sleep(0.2)#allowing the screen to settle (no good reason!)
-            #check for quit request
-            for thisKey in psychopy.event.getKeys():
-                if thisKey in ['q', 'Q', 'escape']:
-                    myWin.close()
-                    return numpy.array([])
+
             #take measurement
             if havePhotom and autoMode=='auto':
                 actualLum = photometer.getLum()
@@ -853,9 +843,25 @@ def getLumSeries(lumLevels=8,
                 else:
                     #otherwise just this gun
                     lumsList[gun,valN] =  actualLum
+
+                #check for quit request
+                for thisKey in psychopy.event.getKeys():
+                    if thisKey in ['q', 'Q', 'escape']:
+                        myWin.close()
+                        return numpy.array([])
+
             elif autoMode=='semi':
                 print "At DAC value %i" % DACval
-                psychopy.event.waitKeys()
+
+                done = False
+                while not done:
+                    #check for quit request
+                    for thisKey in psychopy.event.getKeys():
+                        if thisKey in ['q', 'Q', 'escape']:
+                            myWin.close()
+                            return numpy.array([])
+                        elif thisKey in [' ','space']:
+                            done = True
 
     myWin.close() #we're done with the visual stimuli
     if havePhotom: return lumsList
@@ -871,7 +877,7 @@ def getLumSeriesPR650(lumLevels=8,
     autoMode='auto',
     stimSize = 0.3,
     photometer='COM1'):
-    """DEPRECATED (since v1.60.01): Use :class:`pscyhopy.monitors.getLumSeries()` instead"""
+    """DEPRECATED (since v1.60.01): Use :class:`psychopy.monitors.getLumSeries()` instead"""
 
     logging.warning("DEPRECATED (since v1.60.01): Use monitors.getLumSeries() instead")
     val= getLumSeries(lumLevels,
@@ -936,13 +942,11 @@ def DACrange(n):
 def getAllMonitors():
     """Find the names of all monitors for which calibration files exist
     """
-    currDir = os.getcwd()
-    os.chdir(monitorFolder)
-    monitorList=glob.glob('*.calib')
-    for monitorN,thisName in enumerate(monitorList):
-        monitorList[monitorN] = monitorList[monitorN][:-6]
-
-    os.chdir(currDir)
+    monitorList = glob.glob(os.path.join(monitorFolder, '*.calib'))
+    split = os.path.split
+    splitext = os.path.splitext
+    #skip the folder and the extension for each file
+    monitorList = [splitext(split(thisFile)[-1])[0] for thisFile in monitorList]
     return monitorList
 
 def gammaFun(xx, minLum, maxLum, gamma, eq=1, a=None, b=None, k=None):
@@ -986,19 +990,19 @@ def gammaFun(xx, minLum, maxLum, gamma, eq=1, a=None, b=None, k=None):
     elif eq==3:#NB method 3 was an interpolation method that didn't work well
         pass
     elif eq==4:
-        nMissing = sum([a==None, b==None, k==None])
+        nMissing = sum([a is None, b is None, k is None])
         #check params
         if nMissing>1:
             raise AttributeError, "For eq=4, gammaFun needs 2 of a,b,k to be specified"
         elif nMissing==1:
-            if a==None:
+            if a is None:
                 a = minLum-b**(1.0/gamma)       #when y=min, x=0
-            elif b==None:
+            elif b is None:
                 if a>=minLum:
                     b=0.1**(1.0/gamma)#can't take inv power of -ve
                 else:
                     b = (minLum-a)**(1.0/gamma)     #when y=min, x=0
-            elif k==None:
+            elif k is None:
                 k = (maxLum - a)**(1.0/gamma) - b #when y=max, x=1
         #this is the same as Pelli and Zhang (but different inverse function)
         yy = a+(b+k*xx)**gamma #Pelli and Zhang (1991)
