@@ -1,5 +1,5 @@
 # Part of the PsychoPy library
-# Copyright (C) 2012 Jonathan Peirce
+# Copyright (C) 2014 Jonathan Peirce
 # Distributed under the terms of the GNU General Public License (GPL).
 
 #these are correct for win32, not sure about 64bit versions
@@ -14,33 +14,40 @@
     #define THREAD_PRIORITY_ABOVE_NORMAL    1
     #define THREAD_PRIORITY_HIGHEST         2
     #define THREAD_PRIORITY_TIME_CRITICAL   15
-    
+
+import os
+
 try:
     from ctypes import windll
     windll=windll.kernel32
     importWindllFailed = False
 except:
     importWindllFailed = True
-    log.debug("rush() not available because import windll failed in ext/win32.py")
+    from .. import logging
+    logging.log.debug("rush() not available because import windll failed in ext/win32.py")
+
+FALSE = 0
+
+PROCESS_SET_INFORMATION = 0x0200
+PROCESS_QUERY_INFORMATION = 0x0400
 
 NORMAL_PRIORITY_CLASS    =32
-IDLE_PRIORITY_CLASS      =64
 HIGH_PRIORITY_CLASS      =128
-REALTIME_PRIORITY_CLASS  =1600
-THREAD_PRIORITY_IDLE         =   -15
-THREAD_PRIORITY_LOWEST       =   -2
-THREAD_PRIORITY_BELOW_NORMAL =   -1
+REALTIME_PRIORITY_CLASS  =256
 THREAD_PRIORITY_NORMAL       =   0
-THREAD_PRIORITY_ABOVE_NORMAL =   1
 THREAD_PRIORITY_HIGHEST      =   2
-THREAD_PRIORITY_TIME_CRITICAL=   15
+THREAD_PRIORITY_TIME_CRITICAL =   15
+
+#sleep signals
+ES_CONTINUOUS = 0x80000000
+ES_DISPLAY_REQUIRED = 0x00000002
+ES_SYSTEM_REQUIRED = 0x00000001
+
+def rush(enable=True, realtime=False):
+    """Raise the priority of the current thread/process
 
-def rush(value=True):    
-    """Raise the priority of the current thread/process 
-    Win32 and OS X only so far - on linux use os.nice(niceIncrement)
-    
     Set with rush(True) or rush(False)
-    
+
     Beware and don't take priority until after debugging your code
     and ensuring you have a way out (e.g. an escape sequence of
     keys within the display loop). Otherwise you could end up locked
@@ -48,12 +55,19 @@ def rush(value=True):
     """
     if importWindllFailed:
         return False
-    
-    thr=windll.GetCurrentThread()
-    pr =windll.GetCurrentProcess()
-    if value:
+
+    pr_rights = PROCESS_QUERY_INFORMATION | PROCESS_SET_INFORMATION
+    pr = windll.OpenProcess(pr_rights, FALSE, os.getpid())
+    thr = windll.GetCurrentThread()
+
+    if enable is True:
+        if realtime is False:
+            windll.SetPriorityClass(pr, HIGH_PRIORITY_CLASS)
+            windll.SetThreadPriority(thr, THREAD_PRIORITY_HIGHEST)
+        else:
             windll.SetPriorityClass(pr, REALTIME_PRIORITY_CLASS)
             windll.SetThreadPriority(thr, THREAD_PRIORITY_TIME_CRITICAL)
+
     else:
             windll.SetPriorityClass(pr, NORMAL_PRIORITY_CLASS)
             windll.SetThreadPriority(thr, THREAD_PRIORITY_NORMAL)
@@ -62,4 +76,15 @@ def rush(value=True):
 def waitForVBL():
     """Not implemented on win32 yet
     """
-    pass
\ No newline at end of file
+    pass
+
+def sendStayAwake():
+    """Sends a signal to your system to indicate that the computer is in use and
+    should not sleep. This should be sent periodically, but PsychoPy will send
+    the signal by default on each screen refresh.
+    Added: v1.79.00
+
+    Currently supported on: windows, OS X
+    """
+    success = windll.SetThreadExecutionState( ES_SYSTEM_REQUIRED | ES_DISPLAY_REQUIRED )
+    return success #indicates success
