@@ -1,10 +1,14 @@
-import Image
 from os.path import abspath, basename, dirname, isfile, join as pjoin
 import os.path
 import shutil
 import numpy as np
 from psychopy import logging
 
+try:
+    from PIL import Image
+except ImportError:
+    import Image
+
 try:
     import pytest
     usePytest=True
@@ -41,10 +45,12 @@ def compareScreenshot(fileName, win, crit=5.0):
         expDat = np.array(expected.getdata())
         imgDat = np.array(frame.getdata())
         rms = (((imgDat-expDat)**2).sum()/len(imgDat))**0.5
-        logging.warning('PsychoPyTests: RMS=%.3g at threshold=%3.g'
+        filenameLocal = fileName.replace('.png','_local.png')
+        if rms >= crit/2:
+            #there was SOME discrepency
+            logging.warning('PsychoPyTests: RMS=%.3g at threshold=%3.g'
                   % (rms, crit))
-        if rms>=crit:
-            filenameLocal = fileName.replace('.png','_local.png')
+        if not rms<crit: #don't do `if rms>=crit because that doesn't catch rms=nan
             frame.save(filenameLocal, optimize=1)
             logging.warning('PsychoPyTests: Saving local copy into %s' % filenameLocal)
         assert rms<crit, \
@@ -60,7 +66,7 @@ def compareTextFiles(pathToActual, pathToCorrect, delim=None):
         shutil.copyfile(pathToActual,pathToCorrect)
         assert foundComparisonFile #deliberately raise an error to see the warning message
         return
-    if delim==None:
+    if delim is None:
         if pathToCorrect.endswith('.csv'): delim=','
         elif pathToCorrect.endswith('.dlm'): delim='\t'
 
@@ -70,7 +76,7 @@ def compareTextFiles(pathToActual, pathToCorrect, delim=None):
         txtCorrect = open(pathToCorrect, 'r').readlines()
         assert len(txtActual)==len(txtCorrect), "The data file has the wrong number of lines"
         for lineN in range(len(txtActual)):
-            if delim==None:
+            if delim is None:
                 #just compare the entire line
                 assert lineActual==lineCorrect
             else:#word by word instead
@@ -154,3 +160,29 @@ def compareXlsxFiles(pathToActual, pathToCorrect):
         shutil.copyfile(pathToActual,pathToLocal)
         logging.warning("xlsxActual!=xlsxCorr: Saving local copy to %s" %pathToLocal)
         raise IOError, error
+
+_travisTesting = bool(str(os.environ.get('TRAVIS')).lower() == 'true')  # in Travis-CI testing
+
+# Alternative skip_under_travis implementation;
+# Seems fine, but Jon / Jeremy can decide to use it or loose it.
+#
+# skip_under_travis = pytest.mark.skipif(_travisTesting == True,
+#                                       reason="Cannot be tested under Travis-CI")
+
+def skip_under_travis(fn=None):
+    """Skip if a test is executed under Travis testing environment
+    Could also be used as a decorator (if argument provided) or
+    unparametrized in the code
+    """
+    # TODO: ad-hoc check ATM -- there might be better ways
+    if _travisTesting:
+        skip, msg = pytest.skip, "Cannot be tested under Travis-CI"
+        if fn is not None:
+            def _inner():
+                skip(msg)
+            _inner.__name__ = fn.__name__
+            return _inner
+        else:
+            skip(msg)
+    else:
+        return fn
